@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
@@ -136,7 +137,7 @@ show_error_dialog (GtkWindow   *parent,
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
   					      "%s", detail);
   
-  if (parent->group)
+  if (parent && parent->group)
     gtk_window_group_add_window (parent->group, GTK_WINDOW (dialog));
   
   gtk_dialog_run (GTK_DIALOG (dialog));
@@ -313,6 +314,23 @@ gdict_applet_set_menu_items_sensitive (GdictApplet *applet,
 				NULL);
 }
 
+static gboolean
+window_key_press_event_cb (GtkWidget   *widget,
+			   GdkEventKey *event,
+			   gpointer     user_data)
+{
+  if (event->keyval == GDK_Escape)
+    {
+      GdictApplet *applet = GDICT_APPLET (user_data);
+
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (applet->priv->toggle), FALSE);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 gdict_applet_build_window (GdictApplet *applet)
 {
@@ -331,6 +349,9 @@ gdict_applet_build_window (GdictApplet *applet)
     }
   
   window = gtk_aligned_window_new (priv->toggle);
+  g_signal_connect (window, "key-press-event",
+		    G_CALLBACK (window_key_press_event_cb),
+		    applet);
 
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
@@ -1191,55 +1212,13 @@ static void
 gdict_applet_init (GdictApplet *applet)
 {
   GdictAppletPrivate *priv;
-  gchar *data_dir, *icon_file;
+  gchar *icon_file, *data_dir;
   GdkPixbuf *icon;
   GError *icon_error, *gconf_error;
 
   priv = GDICT_APPLET_GET_PRIVATE (applet);
   applet->priv = priv;
       
-  data_dir = g_build_filename (g_get_home_dir (),
-  			       ".gnome2",
-  			       "gnome-dictionary",
-  			       NULL);
-
-  /* create the data directory inside $HOME, if it doesn't exist yet */
-  if (g_mkdir (data_dir, 0700) == -1)
-    {
-      /* this is weird, but sometimes there's a "gnome-dictionary" file
-       * inside $HOME/.gnome2; see bug #329126.
-       */
-      if ((errno == EEXIST) &&
-	  (g_file_test (data_dir, G_FILE_TEST_IS_REGULAR)))
-	{
-          gchar *message = g_strdup_printf (_("Unable to create the data directory '%s'"), data_dir);
-	  gchar *detail = g_strdup_printf (_("There is a file with the same name in the path.\n"
-			  		     "You should move the file and re-run the\n"
-					     "Dictionary applet"));
-
-	  show_error_dialog (NULL,
-			     message,
-			     detail);
-
-	  g_free (message);
-	  g_free (detail);
-	}
-
-      if (errno != EEXIST)
-        {
-	  gchar *message;
-
-	  message = g_strdup_printf (_("Unable to create the data directory '%s'"),
-				     data_dir);
-	  
-          show_error_dialog (NULL,
-			     message,
-			     strerror (errno));
-
-	  g_free (message);
-	}
-    }
-  
   icon_file = g_build_filename (DATADIR,
 		  		"pixmaps",
 				"gnome-dictionary.png",
@@ -1318,14 +1297,22 @@ gdict_applet_init (GdictApplet *applet)
   if (!priv->loader)
     priv->loader = gdict_source_loader_new ();
 
+  data_dir = g_build_filename (g_get_home_dir (),
+		  	       ".gnome2",
+			       "gnome-dictionary",
+			       NULL);
+
   /* add our data dir inside $HOME to the loader's search paths */
   gdict_source_loader_add_search_path (priv->loader, data_dir);
 
   g_free (data_dir);
 
 #ifndef GDICT_APPLET_STAND_ALONE
+  panel_applet_set_background_widget (PANEL_APPLET (applet),
+				      GTK_WIDGET (applet));
+  
   priv->size = panel_applet_get_size (PANEL_APPLET (applet));
-
+  
   switch (panel_applet_get_orient (PANEL_APPLET (applet)))
     {
     case PANEL_APPLET_ORIENT_LEFT:
@@ -1403,6 +1390,101 @@ gdict_applet_factory (PanelApplet *applet,
 }
 #endif /* !GDICT_APPLET_STAND_ALONE */
 
+/* create the data directory inside $HOME, if it doesn't exist yet */
+static gboolean
+gdict_applet_create_data_dir (void)
+{
+  gchar *data_dir_name;
+  
+  data_dir_name = g_build_filename (g_get_home_dir (),
+  				    ".gnome2",
+  				    "gnome-dictionary",
+  				    NULL);
+  
+  if (g_mkdir (data_dir_name, 0700) == -1)
+    {
+      /* this is weird, but sometimes there's a "gnome-dictionary" file
+       * inside $HOME/.gnome2; see bug #329126.
+       */
+      if ((errno == EEXIST) &&
+          (g_file_test (data_dir_name, G_FILE_TEST_IS_REGULAR)))
+        {
+          gchar *backup = g_strdup_printf ("%s.pre-2-14", data_dir_name);
+	      
+	  if (g_rename (data_dir_name, backup) == -1)
+	    {
+              GtkWidget *error_dialog;
+
+	      error_dialog = gtk_message_dialog_new (NULL,
+                                                     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("Unable to rename file '%s' to '%s': %s"),
+						     data_dir_name,
+						     backup,
+						     g_strerror (errno));
+	      
+	      gtk_dialog_run (GTK_DIALOG (error_dialog));
+	      
+	      gtk_widget_destroy (error_dialog);
+	      g_free (backup);
+	      g_free (data_dir_name);
+
+	      return FALSE;
+            }
+
+	  g_free (backup);
+	  
+          if (g_mkdir (data_dir_name, 0700) == -1)
+            {
+              GtkWidget *error_dialog;
+	      
+	      error_dialog = gtk_message_dialog_new (NULL,
+						     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("Unable to create the data directory '%s': %s"),
+						     data_dir_name,
+						     g_strerror (errno));
+	      
+	      gtk_dialog_run (GTK_DIALOG (error_dialog));
+	      
+	      gtk_widget_destroy (error_dialog);
+              g_free (data_dir_name);
+
+	      return FALSE;
+            }
+
+	  goto success;
+	}
+      
+      if (errno != EEXIST)
+        {
+          GtkWidget *error_dialog;
+
+	  error_dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Unable to create the data directory '%s': %s"),
+						 data_dir_name,
+						 g_strerror (errno));
+	  
+	  gtk_dialog_run (GTK_DIALOG (error_dialog));
+	  
+	  gtk_widget_destroy (error_dialog);
+	  g_free (data_dir_name);
+
+	  return FALSE;
+	}
+    }
+
+success:
+  g_free (data_dir_name);
+
+  return TRUE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1426,6 +1508,9 @@ main (int argc, char *argv[])
 		      GNOME_PROGRAM_STANDARD_PROPERTIES,
 		      NULL);
 
+  if (!gdict_applet_create_data_dir ())
+    return FALSE;
+
   return panel_applet_factory_main ("OAFIID:GNOME_DictionaryApplet_Factory",
 		                    GDICT_TYPE_APPLET,
 				    gdict_applet_factory,
@@ -1433,6 +1518,9 @@ main (int argc, char *argv[])
 #else
   gtk_init (&argc, &argv);
 
+  if (!gdict_applet_create_data_dir ())
+    return FALSE;
+  
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
 
